@@ -3,7 +3,7 @@
  *
  * Create a repository schema.
  *
- * Copyright (c) 2009-2019, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
+ * Copyright (c) 2009-2020, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
  */
 
 -- Adjust this setting to control where the objects get created.
@@ -269,8 +269,10 @@ CREATE TABLE statsrepo.statement
 	userid				oid,
 	queryid				bigint,
 	query				text,
+	plans				bigint,
+	total_plan_time		double precision,
 	calls				bigint,
-	total_time			double precision,
+	total_exec_time		double precision,
 	rows				bigint,
 	shared_blks_hit		bigint,
 	shared_blks_read	bigint,
@@ -349,9 +351,9 @@ CREATE TABLE statsrepo.autovacuum
 	tup_removed				bigint,
 	tup_remain				bigint,
 	tup_dead				bigint,
-	page_hit				integer,
-	page_miss				integer,
-	page_dirty				integer,
+	page_hit				bigint,
+	page_miss				bigint,
+	page_dirty				bigint,
 	read_rate				double precision,
 	write_rate				double precision,
 	duration				real,
@@ -617,7 +619,7 @@ CREATE TABLE statsrepo.stat_subscription
 CREATE TABLE statsrepo.xlog
 (
 	snapid		bigint,
-	location	text,
+	location	pg_lsn,
 	xlogfile	text,
 	FOREIGN KEY (snapid) REFERENCES statsrepo.snapshot (snapid) ON DELETE CASCADE
 );
@@ -668,6 +670,7 @@ CREATE TABLE statsrepo.log
 	user_query_pos		integer,
 	location			text,
 	application_name	text,
+	backend_type		text,
 	FOREIGN KEY (instid) REFERENCES statsrepo.instance (instid) ON DELETE CASCADE
 );
 
@@ -703,7 +706,7 @@ LANGUAGE sql;
 
 -- get_version() - version of statsrepo schema
 CREATE FUNCTION statsrepo.get_version() RETURNS text AS
-'SELECT CAST(''110000'' AS TEXT)'
+'SELECT CAST(''130000'' AS TEXT)'
 LANGUAGE sql IMMUTABLE;
 
 -- tps() - transaction per seconds
@@ -733,27 +736,6 @@ $$
 $$
 LANGUAGE sql IMMUTABLE STRICT;
 
--- xlog_location_diff() - compute the difference in bytes between two WAL locations
-CREATE FUNCTION statsrepo.xlog_location_diff(text, text, bigint)
-RETURNS numeric AS
-$$
-	/* XLogFileSize * (xlogid1 - xlogid2) + xrecoff1 - xrecoff2 */
-	SELECT
-		($3 * (t.xlogid1 - t.xlogid2)::numeric + t.xrecoff1 - t.xrecoff2)
-	FROM
-	(
-		SELECT
-			statsrepo.convert_hex(lsn1[1]) AS xlogid1,
-			statsrepo.convert_hex(lsn1[2]) AS xrecoff1,
-			statsrepo.convert_hex(lsn2[1]) AS xlogid2,
-			statsrepo.convert_hex(lsn2[2]) AS xrecoff2
-		 FROM
-			pg_catalog.regexp_matches($1, '^([0-F]{1,8})/([0-F]{1,8})$') AS lsn1,
-			pg_catalog.regexp_matches($2, '^([0-F]{1,8})/([0-F]{1,8})$') AS lsn2
-	) t;
-$$
-LANGUAGE sql IMMUTABLE STRICT;
-
 -- pg_size_pretty() - formatting with size units
 CREATE FUNCTION statsrepo.pg_size_pretty(bigint)
 RETURNS text AS
@@ -1423,12 +1405,12 @@ $$
 LANGUAGE sql;
 
 -- generate information that corresponds to 'WAL Statistics'
-CREATE FUNCTION statsrepo.get_xlog_tendency(
+CREATE FUNCTION statsrepo.get_wal_tendency(
 	IN snapid_begin			bigint,
 	IN snapid_end			bigint,
 	OUT "timestamp"			text,
-	OUT location			text,
-	OUT xlogfile			text,
+	OUT location			pg_lsn,
+	OUT walfile			text,
 	OUT write_size			numeric,
 	OUT write_size_per_sec	numeric,
 	OUT last_archived_wal	text,
@@ -1452,8 +1434,7 @@ $$
 			s.time,
 			x.location,
 			x.xlogfile,
-			statsrepo.xlog_location_diff(
-				x.location, pg_catalog.lag(x.location) OVER w, i.xlog_file_size) AS write_size,
+			pg_wal_lsn_diff(x.location, pg_catalog.lag(x.location) OVER w) AS write_size,
 			s.time - pg_catalog.lag(s.time) OVER w AS duration,
 			a.last_archived_wal,
 			a.archived_count - pg_catalog.lag(a.archived_count) OVER w AS archive_count,
@@ -1478,7 +1459,7 @@ $$
 LANGUAGE sql;
 
 -- generate information that corresponds to 'WAL Statistics'
-CREATE FUNCTION statsrepo.get_xlog_stats(
+CREATE FUNCTION statsrepo.get_wal_stats(
 	IN snapid_begin			bigint,
 	IN snapid_end			bigint,
 	OUT write_total			numeric,
@@ -1494,10 +1475,10 @@ $$
 		pg_catalog.avg(write_size_per_sec)::numeric(1000, 3),
 		pg_catalog.sum(archive_count),
 		pg_catalog.sum(archive_failed),
-		pg_catalog.max(xlogfile),
+		pg_catalog.max(walfile),
 		pg_catalog.max(last_archived_wal)
 	FROM
-		statsrepo.get_xlog_tendency($1, $2);
+		statsrepo.get_wal_tendency($1, $2);
 $$
 LANGUAGE sql;
 
@@ -2162,7 +2143,7 @@ CREATE FUNCTION statsrepo.get_checkpoint_activity(
 	IN snapid_end		bigint,
 	OUT ckpt_total		bigint,
 	OUT ckpt_time		bigint,
-	OUT ckpt_xlog		bigint,
+	OUT ckpt_wal		bigint,
 	OUT avg_write_buff	numeric,
 	OUT max_write_buff	numeric,
 	OUT avg_duration	numeric,
@@ -2172,7 +2153,7 @@ $$
 	SELECT
 		pg_catalog.count(*),
 		pg_catalog.count(nullif(pg_catalog.strpos(flags, 'time'), 0)),
-		pg_catalog.count(nullif(pg_catalog.strpos(flags, 'xlog'), 0)),
+		pg_catalog.count(nullif(pg_catalog.strpos(flags, 'wal'), 0)),
 		pg_catalog.round(pg_catalog.avg(num_buffers)::numeric,3),
 		pg_catalog.round(pg_catalog.max(num_buffers)::numeric,3),
 		pg_catalog.round(pg_catalog.avg(total_duration)::numeric,3),
@@ -2495,8 +2476,11 @@ CREATE FUNCTION statsrepo.get_query_activity_statements(
 	OUT rolname			text,
 	OUT datname			name,
 	OUT query			text,
+	OUT plans			bigint,
+	OUT total_plan_time	numeric,
+	OUT time_per_plan	numeric,
 	OUT calls			bigint,
-	OUT total_time		numeric,
+	OUT total_exec_time	numeric,
 	OUT time_per_call	numeric,
 	OUT blk_read_time	numeric,
 	OUT blk_write_time	numeric
@@ -2536,8 +2520,11 @@ CREATE FUNCTION statsrepo.get_query_activity_statemets_body(
 	OUT rolname			text,
 	OUT datname			name,
 	OUT query			text,
+	OUT plans			bigint,
+	OUT total_plan_time	numeric,
+	OUT time_per_plan	numeric,
 	OUT calls			bigint,
-	OUT total_time		numeric,
+	OUT total_exec_time	numeric,
 	OUT time_per_call	numeric,
 	OUT blk_read_time	numeric,
 	OUT blk_write_time	numeric
@@ -2547,9 +2534,14 @@ $$
 		t1.rolname::text,
 		t1.dbname::name,
 		t1.query,
+		t1.plans,
+		t1.total_plan_time::numeric(1000,3),
+		CASE t1.plans
+			WHEN 0 THEN 0 ELSE (t1.total_plan_time / t1.plans)::numeric(1000, 3) END,
 		t1.calls,
-		t1.total_time::numeric(1000, 3),
-		(t1.total_time / t1.calls)::numeric(1000, 3),
+		t1.total_exec_time::numeric(1000, 3),
+		CASE t1.calls
+			WHEN 0 THEN 0 ELSE (t1.total_exec_time / t1.calls)::numeric(1000, 3) END,
 		t1.blk_read_time::numeric(1000, 3),
 		t1.blk_write_time::numeric(1000, 3)
 	FROM
@@ -2557,8 +2549,10 @@ $$
 			rol.name AS rolname,
 			db.name AS dbname,
 			reg.query,
+			statsrepo.sub(st2.plans, st1.plans) AS plans,
+			statsrepo.sub(st2.total_plan_time, st1.total_plan_time) AS total_plan_time,
 			statsrepo.sub(st2.calls, st1.calls) AS calls,
-			statsrepo.sub(st2.total_time, st1.total_time) AS total_time,
+			statsrepo.sub(st2.total_exec_time, st1.total_exec_time) AS total_exec_time,
 			statsrepo.sub(st2.blk_read_time, st1.blk_read_time) AS blk_read_time,
 			statsrepo.sub(st2.blk_write_time, st1.blk_write_time) AS blk_write_time
 		 FROM
@@ -2592,10 +2586,10 @@ $$
 			(rol.snapid = reg.first AND rol.userid = reg.userid)
 	) AS t1
 	WHERE
-		t1.calls > 0 and t1.total_time > 0
+		t1.calls > 0 and t1.total_exec_time > 0
 	ORDER BY
-		5 DESC,
-		4 DESC;
+		8 DESC,
+		7 DESC;
 $$
 LANGUAGE sql;
 
@@ -2606,8 +2600,11 @@ CREATE FUNCTION statsrepo.get_query_activity_statemets_body_legacy(
 	OUT rolname			text,
 	OUT datname			name,
 	OUT query			text,
+	OUT plans			bigint,
+	OUT total_plan_time	numeric,
+	OUT time_per_plan	numeric,
 	OUT calls			bigint,
-	OUT total_time		numeric,
+	OUT total_exec_time	numeric,
 	OUT time_per_call	numeric,
 	OUT blk_read_time	numeric,
 	OUT blk_write_time	numeric
@@ -2617,6 +2614,9 @@ $$
 		t1.rolname::text,
 		t1.dbname::name,
 		t1.query,
+		0::bigint,
+		0::numeric(1000, 3),
+		0::numeric(1000, 3),
 		t1.calls,
 		t1.total_time::numeric(1000, 3),
 		(t1.total_time / t1.calls)::numeric(1000, 3),
@@ -2628,7 +2628,7 @@ $$
 			db.name AS dbname,
 			reg.query,
 			statsrepo.sub(st2.calls, st1.calls) AS calls,
-			statsrepo.sub(st2.total_time, st1.total_time) AS total_time,
+			statsrepo.sub(st2.total_exec_time, st1.total_exec_time) AS total_time,
 			statsrepo.sub(st2.blk_read_time, st1.blk_read_time) AS blk_read_time,
 			statsrepo.sub(st2.blk_write_time, st1.blk_write_time) AS blk_write_time
 		 FROM
@@ -2663,8 +2663,8 @@ $$
 	WHERE
 		t1.calls > 0 and t1.total_time > 0
 	ORDER BY
-		5 DESC,
-		4 DESC;
+		8 DESC,
+		7 DESC;
 $$
 LANGUAGE sql;
 
@@ -3017,14 +3017,12 @@ $$
 		r.client_addr,
 		r.application_name,
 		COALESCE(pg_catalog.host(r.client_addr), 'local') || ':' || COALESCE(NULLIF(r.application_name, ''), '(none)') AS client,
-		statsrepo.xlog_location_diff(
-			pg_catalog.split_part(r.current_location, ' ', 1),
-			pg_catalog.split_part(r.flush_location, ' ', 1),
-			i.xlog_file_size),
-		statsrepo.xlog_location_diff(
-			pg_catalog.split_part(r.current_location, ' ', 1),
-			pg_catalog.split_part(r.replay_location, ' ', 1),
-			i.xlog_file_size),
+		pg_wal_lsn_diff(
+			pg_catalog.split_part(r.current_location, ' ', 1)::pg_lsn,
+			pg_catalog.split_part(r.flush_location, ' ', 1)::pg_lsn),
+		pg_wal_lsn_diff(
+			pg_catalog.split_part(r.current_location, ' ', 1)::pg_lsn,
+			pg_catalog.split_part(r.replay_location, ' ', 1)::pg_lsn),
 		r.sync_state
 	FROM
 		statsrepo.replication r LEFT JOIN statsrepo.replication_slots rs
@@ -3106,7 +3104,7 @@ $$
 			client_addr, application_name) d
 		LEFT JOIN statsrepo.replication r
 			ON r.snapid = d.snapid
-			AND d.client_addr = r.client_addr
+			AND ((d.client_addr IS NULL AND r.client_addr IS NULL) OR d.client_addr = r.client_addr)
 			AND d.application_name = r.application_name
 		LEFT JOIN statsrepo.snapshot s ON s.snapid = d.snapid
 	ORDER BY d.snapid DESC, priority_sortkey ASC NULLS LAST;
