@@ -3,7 +3,7 @@
  *
  * Setup of an alert function.
  *
- * Copyright (c) 2009-2019, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
+ * Copyright (c) 2009-2020, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
  */
 
 -- Adjust this setting to control where the objects get created.
@@ -176,13 +176,13 @@ DECLARE
 BEGIN
 	-- calculate the average and maximum of the query-response-time.
 	SELECT
-		pg_catalog.avg((c.total_time - coalesce(p.total_time, 0)) / (c.calls - coalesce(p.calls, 0))),
-		pg_catalog.max((c.total_time - coalesce(p.total_time, 0)) / (c.calls - coalesce(p.calls, 0)))
+		pg_catalog.avg((c.total_exec_time - coalesce(p.total_exec_time, 0)) / (c.calls - coalesce(p.calls, 0))),
+		pg_catalog.max((c.total_exec_time - coalesce(p.total_exec_time, 0)) / (c.calls - coalesce(p.calls, 0)))
 	INTO val_res_avg, val_res_max
-	FROM (SELECT dbid, userid, total_time, calls, query
+	FROM (SELECT dbid, userid, total_exec_time, calls, query
 			FROM statsrepo.statement WHERE snapid = $1.snapid) AS c
 		 LEFT OUTER JOIN
-		 (SELECT dbid, userid, total_time, calls, query
+		 (SELECT dbid, userid, total_exec_time, calls, query
 		 	FROM statsrepo.statement WHERE snapid = $2.snapid) AS p
 		 ON c.dbid = p.dbid AND c.userid = p.userid AND c.query = p.query
 	WHERE c.calls <> coalesce(p.calls, 0);
@@ -332,19 +332,15 @@ BEGIN
 		SELECT
 			pg_catalog.host(r.client_addr) || ':' || r.client_port,
 			statsrepo.div(
-				statsrepo.xlog_location_diff(
-					pg_catalog.split_part(r.current_location, ' ', 1),
-					pg_catalog.split_part(r.flush_location, ' ', 1),
-					i.xlog_file_size
-				),
+				pg_wal_lsn_diff(
+					pg_catalog.split_part(r.current_location, ' ', 1)::pg_lsn,
+					pg_catalog.split_part(r.flush_location, ' ', 1)::pg_lsn),
 				1024 * 1024
 			),
 			statsrepo.div(
-				statsrepo.xlog_location_diff(
-					pg_catalog.split_part(r.current_location, ' ', 1),
-					pg_catalog.split_part(r.replay_location, ' ', 1),
-					i.xlog_file_size
-				),
+				pg_wal_lsn_diff(
+					pg_catalog.split_part(r.current_location, ' ', 1)::pg_lsn,
+					pg_catalog.split_part(r.replay_location, ' ', 1)::pg_lsn),
 				1024 * 1024
 			)
 		FROM
